@@ -21,10 +21,12 @@ import errno
 import os
 import posixpath
 from stat import S_IEXEC, S_ISREG
+import tempfile
 import time
 
 from .mapping import encode_git_path, mode_kind, mode_is_executable, object_mode
 from .tree import GitTree, GitTreeDirectory, GitTreeSymlink, GitTreeFile
+from .mapping import decode_git_path
 
 from .. import (
     annotate,
@@ -34,6 +36,7 @@ from .. import (
     osutils,
     revision as _mod_revision,
     trace,
+    transport as _mod_transport,
     ui,
     urlutils,
     )
@@ -246,7 +249,7 @@ class TreeTransformBase(TreeTransform):
                 try:
                     if self._tree.stored_kind(path) == 'directory':
                         parents.append(trans_id)
-                except errors.NoSuchFile:
+                except _mod_transport.NoSuchFile:
                     pass
             elif self.tree_kind(trans_id) == 'directory':
                 parents.append(trans_id)
@@ -674,7 +677,7 @@ class TreeTransformBase(TreeTransform):
         try:
             if path is None or self._tree.kind(path) != 'file':
                 return None
-        except errors.NoSuchFile:
+        except _mod_transport.NoSuchFile:
             return None
         return path
 
@@ -723,6 +726,9 @@ class TreeTransformBase(TreeTransform):
         """
         raise NotImplementedError(self.create_symlink)
 
+    def create_tree_reference(self, target, trans_id):
+        raise NotImplementedError(self.create_tree_reference)
+
     def create_hardlink(self, path, trans_id):
         """Schedule creation of a hard link"""
         raise NotImplementedError(self.create_hardlink)
@@ -990,6 +996,16 @@ class DiskTreeTransform(TreeTransformBase):
         # conflicts on platforms that don't support symlink
         unique_add(self._new_contents, trans_id, 'symlink')
 
+    def create_tree_reference(self, reference_revision, trans_id):
+        """Schedule creation of a new symbolic link.
+
+        target is a bytestring.
+        See also new_symlink.
+        """
+        os.mkdir(self._limbo_name(trans_id))
+        unique_add(self._new_reference_revision, trans_id, reference_revision)
+        unique_add(self._new_contents, trans_id, 'tree-reference')
+
     def cancel_creation(self, trans_id):
         """Cancel the creation of new file contents."""
         del self._new_contents[trans_id]
@@ -1215,7 +1231,7 @@ class GitTreeTransform(DiskTreeTransform):
             return None
         try:
             return osutils.file_kind(self._tree.abspath(path))
-        except errors.NoSuchFile:
+        except _mod_transport.NoSuchFile:
             return None
 
     def _set_mode(self, trans_id, mode_id, typefunc):
@@ -1425,6 +1441,22 @@ class GitTreeTransform(DiskTreeTransform):
                     o_sha1, o_st_val = self._observed_sha1s[trans_id]
                     st = osutils.lstat(full_path)
                     self._observed_sha1s[trans_id] = (o_sha1, st)
+                if trans_id in self._new_reference_revision:
+                    for (submodule_path, submodule_url, submodule_name) in self._tree._submodule_config():
+                        if decode_git_path(submodule_path) == path:
+                            break
+                    else:
+                        trace.warning(
+                            'unable to find submodule for path %s', path)
+                        continue
+                    submodule_transport = self._tree.controldir.control_transport.clone(
+                        os.path.join('modules', submodule_name.decode('utf-8')))
+                    submodule_transport.create_prefix()
+                    from .dir import BareLocalGitControlDirFormat
+                    BareLocalGitControlDirFormat().initialize_on_transport(submodule_transport)
+                    with open(os.path.join(full_path, '.git'), 'w') as f:
+                        submodule_abspath = submodule_transport.local_abspath('.')
+                        f.write('gitdir: %s\n' % os.path.relpath(submodule_abspath, full_path))
         for path, trans_id in new_paths:
             # new_paths includes stuff like workingtree conflicts. Only the
             # stuff in new_contents actually comes from limbo.
@@ -1497,7 +1529,7 @@ class GitTransformPreview(GitTreeTransform):
 
     def __init__(self, tree, pb=None, case_sensitive=True):
         tree.lock_read()
-        limbodir = osutils.mkdtemp(prefix='git-limbo-')
+        limbodir = tempfile.mkdtemp(prefix='git-limbo-')
         DiskTreeTransform.__init__(self, tree, limbodir, pb, case_sensitive)
 
     def canonical_path(self, path):
@@ -1531,7 +1563,7 @@ class GitTransformPreview(GitTreeTransform):
             for child in self._tree.iter_child_entries(path):
                 childpath = joinpath(path, child.name)
                 yield self.trans_id_tree_path(childpath)
-        except errors.NoSuchFile:
+        except _mod_transport.NoSuchFile:
             return
 
     def new_orphan(self, trans_id, parent_id):
@@ -1541,6 +1573,8 @@ class GitTransformPreview(GitTreeTransform):
 class GitPreviewTree(PreviewTree, GitTree):
     """Partial implementation of Tree to support show_diff_trees"""
 
+    supports_file_ids = False
+
     def __init__(self, transform):
         PreviewTree.__init__(self, transform)
         self.store = transform._tree.store
@@ -1550,6 +1584,9 @@ class GitPreviewTree(PreviewTree, GitTree):
     def supports_setting_file_ids(self):
         return False
 
+    def supports_symlinks(self):
+        return self._transform._create_symlinks
+
     def _supports_executable(self):
         return self._transform._limbo_supports_executable()
 
@@ -1601,12 +1638,12 @@ class GitPreviewTree(PreviewTree, GitTree):
         """See Tree.get_file"""
         trans_id = self._path2trans_id(path)
         if trans_id is None:
-            raise errors.NoSuchFile(path)
+            raise _mod_transport.NoSuchFile(path)
         if trans_id in self._transform._new_contents:
             name = self._transform._limbo_name(trans_id)
             return open(name, 'rb')
         if trans_id in self._transform._removed_contents:
-            raise errors.NoSuchFile(path)
+            raise _mod_transport.NoSuchFile(path)
         orig_path = self._transform.tree_path(trans_id)
         return self._transform._tree.get_file(orig_path)
 
@@ -1614,7 +1651,7 @@ class GitPreviewTree(PreviewTree, GitTree):
         """See Tree.get_symlink_target"""
         trans_id = self._path2trans_id(path)
         if trans_id is None:
-            raise errors.NoSuchFile(path)
+            raise _mod_transport.NoSuchFile(path)
         if trans_id not in self._transform._new_contents:
             orig_path = self._transform.tree_path(trans_id)
             return self._transform._tree.get_symlink_target(orig_path)
@@ -1633,10 +1670,19 @@ class GitPreviewTree(PreviewTree, GitTree):
             old_annotation = []
         try:
             lines = self.get_file_lines(path)
-        except errors.NoSuchFile:
+        except _mod_transport.NoSuchFile:
             return None
         return annotate.reannotate([old_annotation], lines, default_revision)
 
+    def path2id(self, path):
+        if isinstance(path, list):
+            if path == []:
+                path = [""]
+            path = osutils.pathjoin(*path)
+        if not self.is_versioned(path):
+            return None
+        return self._transform._tree.mapping.generate_file_id(path)
+
     def get_file_text(self, path):
         """Return the byte content of a file.
 
@@ -1691,7 +1737,7 @@ class GitPreviewTree(PreviewTree, GitTree):
             if kind == 'symlink':
                 link_or_sha1 = os.readlink(limbo_name)
                 if not isinstance(link_or_sha1, str):
-                    link_or_sha1 = link_or_sha1.decode(osutils._fs_enc)
+                    link_or_sha1 = os.fsdecode(link_or_sha1)
         executable = tt._new_executability.get(trans_id, executable)
         return kind, size, executable, link_or_sha1
 
@@ -1699,7 +1745,7 @@ class GitPreviewTree(PreviewTree, GitTree):
         """See Tree.get_file_mtime"""
         trans_id = self._path2trans_id(path)
         if trans_id is None:
-            raise errors.NoSuchFile(path)
+            raise _mod_transport.NoSuchFile(path)
         if trans_id not in self._transform._new_contents:
             return self._transform._tree.get_file_mtime(
                 self._transform.tree_path(trans_id))
@@ -1777,7 +1823,7 @@ class GitPreviewTree(PreviewTree, GitTree):
     def iter_child_entries(self, path):
         trans_id = self._path2trans_id(path)
         if trans_id is None:
-            raise errors.NoSuchFile(path)
+            raise _mod_transport.NoSuchFile(path)
         for child_trans_id in self._all_children(trans_id):
             entry, is_versioned = self._transform.final_entry(trans_id)
             if not is_versioned:
