@@ -14,8 +14,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-from __future__ import absolute_import
-
 import errno
 import os
 import subprocess
@@ -32,10 +30,6 @@ from . import (
     urlutils,
     registry,
     )
-from .sixish import (
-    PY3,
-    text_type,
-    )
 
 mail_client_registry = registry.Registry()
 
@@ -170,13 +164,10 @@ class BodyExternalMailClient(MailClient):
         """
         if basename is None:
             basename = 'attachment'
-        pathname = osutils.mkdtemp(prefix='bzr-mail-')
+        pathname = tempfile.mkdtemp(prefix='bzr-mail-')
         attach_path = osutils.pathjoin(pathname, basename + extension)
-        outfile = open(attach_path, 'wb')
-        try:
+        with open(attach_path, 'wb') as outfile:
             outfile.write(attachment)
-        finally:
-            outfile.close()
         if body is not None:
             kwargs = {'body': body}
         else:
@@ -238,8 +229,6 @@ class BodyExternalMailClient(MailClient):
         :param  u:  possible unicode string.
         :return:    encoded string if u is unicode, u itself otherwise.
         """
-        if not PY3 and isinstance(u, text_type):
-            return u.encode(osutils.get_user_encoding(), 'replace')
         return u
 
     def _encode_path(self, path, kind):
@@ -251,11 +240,6 @@ class BodyExternalMailClient(MailClient):
                         path itself otherwise.
         :raise:         UnableEncodePath.
         """
-        if not PY3 and isinstance(path, text_type):
-            try:
-                return path.encode(osutils.get_user_encoding())
-            except UnicodeEncodeError:
-                raise errors.UnableEncodePath(path, kind)
         return path
 
 
