@@ -15,24 +15,17 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 
-from .lazy_import import lazy_import
-lazy_import(globals(), """
-import bisect
-import datetime
-
 from breezy import (
-    branch as _mod_branch,
-    cache_utf8,
     revision,
     workingtree,
     )
 from breezy.i18n import gettext
-""")
 
 from . import (
     errors,
     lazy_regex,
     registry,
+    revision as _mod_revision,
     trace,
     )
 
@@ -422,8 +415,9 @@ class RevisionSpec_revno(RevisionSpec):
                 dotted = True
 
         if branch_spec:
+            from .branch import Branch
             # the user has overriden the branch to look in.
-            branch = _mod_branch.Branch.open(branch_spec)
+            branch = Branch.open(branch_spec)
 
         if dotted:
             try:
@@ -496,7 +490,7 @@ class RevisionSpec_revid(RevisionIDSpec):
         # so we expect it to be a Unicode string. Switch it to the internal
         # representation.
         if isinstance(self.spec, str):
-            return cache_utf8.encode(self.spec)
+            return self.spec.encode('utf-8')
         return self.spec
 
 
@@ -649,11 +643,55 @@ class _RevListToTimestamps(object):
     def __getitem__(self, index):
         """Get the date of the index'd item"""
         r = self.branch.repository.get_revision(self.branch.get_rev_id(index))
-        # TODO: Handle timezone.
-        return datetime.datetime.fromtimestamp(r.timestamp)
+        return r.datetime()
 
-    def __len__(self):
-        return self.branch.revno()
+
+_date_regex = lazy_regex.lazy_compile(
+    r'(?P<date>(?P<year>\d\d\d\d)-(?P<month>\d\d)-(?P<day>\d\d))?'
+    r'(,|T)?\s*'
+    r'(?P<time>(?P<hour>\d\d):(?P<minute>\d\d)(:(?P<second>\d\d))?)?'
+    )
+
+
+def _parse_datespec(spec):
+    import datetime
+    #  XXX: This doesn't actually work
+    #  So the proper way of saying 'give me all entries for today' is:
+    #      -r date:yesterday..date:today
+    today = datetime.datetime.fromordinal(
+        datetime.date.today().toordinal())
+    if spec.lower() == 'yesterday':
+        return today - datetime.timedelta(days=1)
+    elif spec.lower() == 'today':
+        return today
+    elif spec.lower() == 'tomorrow':
+        return today + datetime.timedelta(days=1)
+    else:
+        m = _date_regex.match(spec)
+        if not m or (not m.group('date') and not m.group('time')):
+            raise ValueError
+
+        if m.group('date'):
+            year = int(m.group('year'))
+            month = int(m.group('month'))
+            day = int(m.group('day'))
+        else:
+            year = today.year
+            month = today.month
+            day = today.day
+
+        if m.group('time'):
+            hour = int(m.group('hour'))
+            minute = int(m.group('minute'))
+            if m.group('second'):
+                second = int(m.group('second'))
+            else:
+                second = 0
+        else:
+            hour, minute, second = 0, 0, 0
+
+        return datetime.datetime(year=year, month=month, day=day,
+                               hour=hour, minute=minute, second=second)
 
 
 class RevisionSpec_date(RevisionSpec):
@@ -677,11 +715,28 @@ class RevisionSpec_date(RevisionSpec):
                                    August 14th, 2006 at 5:10pm.
     """
     prefix = 'date:'
-    _date_regex = lazy_regex.lazy_compile(
-        r'(?P<date>(?P<year>\d\d\d\d)-(?P<month>\d\d)-(?P<day>\d\d))?'
-        r'(,|T)?\s*'
-        r'(?P<time>(?P<hour>\d\d):(?P<minute>\d\d)(:(?P<second>\d\d))?)?'
-        )
+
+    def _scan_backwards(self, branch, dt):
+        with branch.lock_read():
+            graph = branch.repository.get_graph()
+            last_match = None
+            for revid in graph.iter_lefthand_ancestry(
+                    branch.last_revision(), (_mod_revision.NULL_REVISION,)):
+                r = branch.repository.get_revision(revid)
+                if r.datetime() < dt:
+                    if last_match is None:
+                        raise InvalidRevisionSpec(self.user_spec, branch)
+                    return RevisionInfo(branch, None, last_match)
+                last_match = revid
+            return RevisionInfo(branch, None, last_match)
+
+    def _bisect_backwards(self, branch, dt, hi):
+        import bisect
+        with branch.lock_read():
+            rev = bisect.bisect(_RevListToTimestamps(branch), dt, 1, hi)
+        if rev == branch.revno():
+            raise InvalidRevisionSpec(self.user_spec, branch)
+        return RevisionInfo(branch, rev)
 
     def _match_on(self, branch, revs):
         """Spec for date revisions:
@@ -690,53 +745,16 @@ class RevisionSpec_date(RevisionSpec):
           matches the first entry after a given date (either at midnight or
           at a specified time).
         """
-        #  XXX: This doesn't actually work
-        #  So the proper way of saying 'give me all entries for today' is:
-        #      -r date:yesterday..date:today
-        today = datetime.datetime.fromordinal(
-            datetime.date.today().toordinal())
-        if self.spec.lower() == 'yesterday':
-            dt = today - datetime.timedelta(days=1)
-        elif self.spec.lower() == 'today':
-            dt = today
-        elif self.spec.lower() == 'tomorrow':
-            dt = today + datetime.timedelta(days=1)
+        try:
+            dt = _parse_datespec(self.spec)
+        except ValueError:
+            raise InvalidRevisionSpec(
+                self.user_spec, branch, 'invalid date')
+        revno = branch.revno()
+        if revno is None:
+            return self._scan_backwards(branch, dt)
         else:
-            m = self._date_regex.match(self.spec)
-            if not m or (not m.group('date') and not m.group('time')):
-                raise InvalidRevisionSpec(
-                    self.user_spec, branch, 'invalid date')
-
-            try:
-                if m.group('date'):
-                    year = int(m.group('year'))
-                    month = int(m.group('month'))
-                    day = int(m.group('day'))
-                else:
-                    year = today.year
-                    month = today.month
-                    day = today.day
-
-                if m.group('time'):
-                    hour = int(m.group('hour'))
-                    minute = int(m.group('minute'))
-                    if m.group('second'):
-                        second = int(m.group('second'))
-                    else:
-                        second = 0
-                else:
-                    hour, minute, second = 0, 0, 0
-            except ValueError:
-                raise InvalidRevisionSpec(
-                    self.user_spec, branch, 'invalid date')
-
-            dt = datetime.datetime(year=year, month=month, day=day,
-                                   hour=hour, minute=minute, second=second)
-        with branch.lock_read():
-            rev = bisect.bisect(_RevListToTimestamps(branch), dt, 1)
-        if rev == branch.revno():
-            raise InvalidRevisionSpec(self.user_spec, branch)
-        return RevisionInfo(branch, rev)
+            return self._bisect_backwards(branch, dt, revno)
 
 
 class RevisionSpec_ancestor(RevisionSpec):
@@ -950,7 +968,8 @@ class RevisionSpec_mainline(RevisionIDSpec):
         if revspec.get_branch() is None:
             spec_branch = context_branch
         else:
-            spec_branch = _mod_branch.Branch.open(revspec.get_branch())
+            from .branch import Branch
+            spec_branch = Branch.open(revspec.get_branch())
         revision_id = revspec.as_revision_id(spec_branch)
         graph = context_branch.repository.get_graph()
         result = graph.find_lefthand_merger(revision_id,
