@@ -17,38 +17,14 @@
 """Tree classes, representing directory at point in time.
 """
 
-from __future__ import absolute_import
-
-try:
-    from collections.abc import deque
-except ImportError:  # python < 3.7
-    from collections import deque
-
-from .lazy_import import lazy_import
-lazy_import(globals(), """
-
-from breezy import (
-    conflicts as _mod_conflicts,
-    debug,
-    delta,
-    filters,
-    revision as _mod_revision,
-    rules,
-    trace,
-    )
-from breezy.i18n import gettext
-""")
-
 from . import (
     errors,
     lock,
     osutils,
+    revision as _mod_revision,
+    trace,
     )
 from .inter import InterObject
-from .sixish import (
-    text_type,
-    viewvalues,
-    )
 
 
 class FileTimestampUnavailable(errors.BzrError):
@@ -141,16 +117,14 @@ class TreeReference(TreeEntry):
 class TreeChange(object):
     """Describes the changes between the same item in two different trees."""
 
-    __slots__ = ['file_id', 'path', 'changed_content', 'versioned', 'parent_id',
+    __slots__ = ['path', 'changed_content', 'versioned',
                  'name', 'kind', 'executable', 'copied']
 
-    def __init__(self, file_id, path, changed_content, versioned, parent_id,
+    def __init__(self, path, changed_content, versioned,
                  name, kind, executable, copied=False):
-        self.file_id = file_id
         self.path = path
         self.changed_content = changed_content
         self.versioned = versioned
-        self.parent_id = parent_id
         self.name = name
         self.kind = kind
         self.executable = executable
@@ -159,12 +133,9 @@ class TreeChange(object):
     def __repr__(self):
         return "%s%r" % (self.__class__.__name__, self._as_tuple())
 
-    def __len__(self):
-        return len(self.__slots__)
-
     def _as_tuple(self):
-        return (self.file_id, self.path, self.changed_content, self.versioned,
-                self.parent_id, self.name, self.kind, self.executable, self.copied)
+        return (self.path, self.changed_content, self.versioned,
+                self.name, self.kind, self.executable, self.copied)
 
     def __eq__(self, other):
         if isinstance(other, TreeChange):
@@ -181,13 +152,20 @@ class TreeChange(object):
             return (self.executable[0] != self.executable[1])
         return False
 
+    @property
+    def renamed(self):
+        return (
+            not self.copied and
+            None not in self.name and
+            self.path[0] != self.path[1])
+
     def is_reparented(self):
-        return self.parent_id[0] != self.parent_id[1]
+        return osutils.dirname(self.path[0]) != osutils.dirname(self.path[1])
 
     def discard_new(self):
         return self.__class__(
-            self.file_id, (self.path[0], None), self.changed_content,
-            (self.versioned[0], None), (self.parent_id[0], None),
+            (self.path[0], None), self.changed_content,
+            (self.versioned[0], None),
             (self.name[0], None), (self.kind[0], None),
             (self.executable[0], None),
             copied=False)
@@ -225,7 +203,13 @@ class Tree(object):
     def supports_symlinks(self):
         """Does this tree support symbolic links?
         """
-        return osutils.has_symlinks()
+        return True
+
+    @property
+    def supports_file_ids(self):
+        """Does this tree support file ids?
+        """
+        raise NotImplementedError(self.supports_file_ids)
 
     def changes_from(self, other, want_unchanged=False, specific_files=None,
                      extra_trees=None, require_versioned=False, include_root=False,
@@ -275,6 +259,7 @@ class Tree(object):
 
         Each conflict is an instance of breezy.conflicts.Conflict.
         """
+        from . import conflicts as _mod_conflicts
         return _mod_conflicts.ConflictList()
 
     def extras(self):
@@ -302,21 +287,10 @@ class Tree(object):
         """
         return False
 
-    def all_file_ids(self):
-        """Iterate through all file ids, including ids for missing files."""
-        raise NotImplementedError(self.all_file_ids)
-
     def all_versioned_paths(self):
         """Iterate through all paths, including paths for missing files."""
         raise NotImplementedError(self.all_versioned_paths)
 
-    def id2path(self, file_id, recurse='down'):
-        """Return the path for a file id.
-
-        :raises NoSuchId:
-        """
-        raise NotImplementedError(self.id2path)
-
     def iter_entries_by_dir(self, specific_files=None, recurse_nested=False):
         """Walk the tree in 'by_dir' order.
 
@@ -575,17 +549,13 @@ class Tree(object):
         """
         raise NotImplementedError(self.annotate_iter)
 
-    def path2id(self, path):
-        """Return the id for path in this tree."""
-        raise NotImplementedError(self.path2id)
-
     def is_versioned(self, path):
         """Check whether path is versioned.
 
         :param path: Path to check
         :return: boolean
         """
-        return self.path2id(path) is not None
+        raise NotImplementedError(self.is_versioned)
 
     def find_related_paths_across_trees(self, paths, trees=[],
                                         require_versioned=True):
@@ -655,12 +625,10 @@ class Tree(object):
         list to exclude some directories, they are then not descended into.
 
         The data yielded is of the form:
-        ((directory-relpath, directory-path-from-root, directory-fileid),
-        [(relpath, basename, kind, lstat, path_from_tree_root, file_id,
+        (directory-relpath,
+        [(relpath, basename, kind, lstat, path_from_tree_root,
           versioned_kind), ...]),
-         - directory-relpath is the containing dirs relpath from prefix
          - directory-path-from-root is the containing dirs path from /
-         - directory-fileid is the id of the directory if it is versioned.
          - relpath is the relative path within the subtree being walked.
          - basename is the basename
          - kind is the kind of the file now. If unknonwn then the file is not
@@ -668,7 +636,6 @@ class Tree(object):
            versioned_kind.
          - lstat is the stat data *if* the file was statted.
          - path_from_tree_root is the path from the root of the tree.
-         - file_id is the file_id if the entry is versioned.
          - versioned_kind is the kind of the file as last recorded in the
            versioning system. If 'unknown' the file is not versioned.
         One of 'kind' and 'versioned_kind' must not be 'unknown'.
@@ -694,14 +661,14 @@ class Tree(object):
             or None if unknown
         :return: the list of filters - [] if there are none
         """
+        from . import debug, filters
         filter_pref_names = filters._get_registered_names()
         if len(filter_pref_names) == 0:
             return []
         prefs = next(self.iter_search_rules([path], filter_pref_names))
         stk = filters._get_filter_stack_for(prefs)
         if 'filters' in debug.debug_flags:
-            trace.note(
-                gettext("*** {0} content-filter: {1} => {2!r}").format(path, prefs, stk))
+            trace.note("*** {0} content-filter: {1} => {2!r}").format(path, prefs, stk)
         return stk
 
     def _content_filter_stack_provider(self):
@@ -728,6 +695,7 @@ class Tree(object):
         :return: an iterator of tuple sequences, one per path-name.
           See _RulesSearcher.get_items for details on the tuple sequence.
         """
+        from . import rules
         if _default_searcher is None:
             _default_searcher = rules._per_user_searcher
         searcher = self._get_rules_searcher(_default_searcher)
@@ -745,25 +713,31 @@ class Tree(object):
         return searcher
 
     def archive(self, format, name, root='', subdir=None,
-                force_mtime=None):
+                force_mtime=None, recurse_nested=False):
         """Create an archive of this tree.
 
-        :param format: Format name (e.g. 'tar')
-        :param name: target file name
-        :param root: Root directory name (or None)
-        :param subdir: Subdirectory to export (or None)
-        :return: Iterator over archive chunks
+        Args:
+          format: Format name (e.g. 'tar')
+          name: target file name
+          root: Root directory name (or None)
+          subdir: Subdirectory to export (or None)
+        Returns: Iterator over archive chunks
         """
         from .archive import create_archive
         with self.lock_read():
             return create_archive(format, self, name, root,
-                                  subdir, force_mtime=force_mtime)
+                                  subdir, force_mtime=force_mtime,
+                                  recurse_nested=recurse_nested)
 
     @classmethod
     def versionable_kind(cls, kind):
         """Check if this tree support versioning a specific file kind."""
         return (kind in ('file', 'directory', 'symlink', 'tree-reference'))
 
+    def preview_transform(self, pb=None):
+        """Obtain a transform object."""
+        raise NotImplementedError(self.preview_transform)
+
 
 class InterTree(InterObject):
     """This class represents operations taking place between two Trees.
@@ -790,78 +764,6 @@ class InterTree(InterObject):
         # it works for all trees.
         return True
 
-    def _changes_from_entries(self, source_entry, target_entry, source_path,
-                              target_path):
-        """Generate a iter_changes tuple between source_entry and target_entry.
-
-        :param source_entry: An inventory entry from self.source, or None.
-        :param target_entry: An inventory entry from self.target, or None.
-        :param source_path: The path of source_entry.
-        :param target_path: The path of target_entry.
-        :return: A tuple, item 0 of which is an iter_changes result tuple, and
-            item 1 is True if there are any changes in the result tuple.
-        """
-        if source_entry is None:
-            if target_entry is None:
-                return None
-            file_id = target_entry.file_id
-        else:
-            file_id = source_entry.file_id
-        if source_entry is not None:
-            source_versioned = True
-            source_name = source_entry.name
-            source_parent = source_entry.parent_id
-            source_kind, source_executable, source_stat = \
-                self.source._comparison_data(source_entry, source_path)
-        else:
-            source_versioned = False
-            source_name = None
-            source_parent = None
-            source_kind = None
-            source_executable = None
-        if target_entry is not None:
-            target_versioned = True
-            target_name = target_entry.name
-            target_parent = target_entry.parent_id
-            target_kind, target_executable, target_stat = \
-                self.target._comparison_data(target_entry, target_path)
-        else:
-            target_versioned = False
-            target_name = None
-            target_parent = None
-            target_kind = None
-            target_executable = None
-        versioned = (source_versioned, target_versioned)
-        kind = (source_kind, target_kind)
-        changed_content = False
-        if source_kind != target_kind:
-            changed_content = True
-        elif source_kind == 'file':
-            if not self.file_content_matches(
-                    source_path, target_path,
-                    source_stat, target_stat):
-                changed_content = True
-        elif source_kind == 'symlink':
-            if (self.source.get_symlink_target(source_path) !=
-                    self.target.get_symlink_target(target_path)):
-                changed_content = True
-        elif source_kind == 'tree-reference':
-            if (self.source.get_reference_revision(source_path)
-                    != self.target.get_reference_revision(target_path)):
-                changed_content = True
-        parent = (source_parent, target_parent)
-        name = (source_name, target_name)
-        executable = (source_executable, target_executable)
-        if (changed_content is not False or versioned[0] != versioned[1] or
-            parent[0] != parent[1] or name[0] != name[1] or
-                executable[0] != executable[1]):
-            changes = True
-        else:
-            changes = False
-        return TreeChange(
-            file_id, (source_path, target_path), changed_content,
-            versioned, parent, name, kind, executable), changes
-
     def compare(self, want_unchanged=False, specific_files=None,
                 extra_trees=None, require_versioned=False, include_root=False,
                 want_unversioned=False):
@@ -881,6 +783,7 @@ class InterTree(InterObject):
             a PathsNotVersionedError will be thrown.
         :param want_unversioned: Scan for unversioned paths.
         """
+        from . import delta
         trees = (self.source,)
         if extra_trees is not None:
             trees = trees + tuple(extra_trees)
@@ -895,10 +798,7 @@ class InterTree(InterObject):
                      require_versioned=True, want_unversioned=False):
         """Generate an iterator of changes between trees.
 
-        A tuple is returned:
-        (file_id, (path_in_source, path_in_target),
-         changed_content, versioned, parent, name, kind,
-         executable)
+        A TreeChange object is returned.
 
         Changed_content is True if the file's content has changed.  This
         includes changes to its kind, and to a symlink's target.
@@ -926,237 +826,7 @@ class InterTree(InterObject):
             output. An unversioned file is defined as one with (False, False)
             for the versioned pair.
         """
-        if not extra_trees:
-            extra_trees = []
-        else:
-            extra_trees = list(extra_trees)
-        # The ids of items we need to examine to insure delta consistency.
-        precise_file_ids = set()
-        changed_file_ids = []
-        if specific_files == []:
-            target_specific_files = []
-            source_specific_files = []
-        else:
-            target_specific_files = self.target.find_related_paths_across_trees(
-                specific_files, [self.source] + extra_trees,
-                require_versioned=require_versioned)
-            source_specific_files = self.source.find_related_paths_across_trees(
-                specific_files, [self.target] + extra_trees,
-                require_versioned=require_versioned)
-        if specific_files is not None:
-            # reparented or added entries must have their parents included
-            # so that valid deltas can be created. The seen_parents set
-            # tracks the parents that we need to have.
-            # The seen_dirs set tracks directory entries we've yielded.
-            # After outputting version object in to_entries we set difference
-            # the two seen sets and start checking parents.
-            seen_parents = set()
-            seen_dirs = set()
-        if want_unversioned:
-            all_unversioned = sorted([(p.split('/'), p) for p in
-                                      self.target.extras()
-                                      if specific_files is None or
-                                      osutils.is_inside_any(specific_files, p)])
-            all_unversioned = deque(all_unversioned)
-        else:
-            all_unversioned = deque()
-        to_paths = {}
-        from_entries_by_dir = list(self.source.iter_entries_by_dir(
-            specific_files=source_specific_files))
-        from_data = dict(from_entries_by_dir)
-        to_entries_by_dir = list(self.target.iter_entries_by_dir(
-            specific_files=target_specific_files))
-        path_equivs = self.find_source_paths([p for p, e in to_entries_by_dir])
-        num_entries = len(from_entries_by_dir) + len(to_entries_by_dir)
-        entry_count = 0
-        # the unversioned path lookup only occurs on real trees - where there
-        # can be extras. So the fake_entry is solely used to look up
-        # executable it values when execute is not supported.
-        fake_entry = TreeFile()
-        for target_path, target_entry in to_entries_by_dir:
-            while (all_unversioned and
-                   all_unversioned[0][0] < target_path.split('/')):
-                unversioned_path = all_unversioned.popleft()
-                target_kind, target_executable, target_stat = \
-                    self.target._comparison_data(
-                        fake_entry, unversioned_path[1])
-                yield TreeChange(
-                    None, (None, unversioned_path[1]), True, (False, False),
-                    (None, None),
-                    (None, unversioned_path[0][-1]),
-                    (None, target_kind),
-                    (None, target_executable))
-            source_path = path_equivs[target_path]
-            if source_path is not None:
-                source_entry = from_data.get(source_path)
-            else:
-                source_entry = None
-            result, changes = self._changes_from_entries(
-                source_entry, target_entry, source_path=source_path, target_path=target_path)
-            to_paths[result.file_id] = result.path[1]
-            entry_count += 1
-            if result.versioned[0]:
-                entry_count += 1
-            if pb is not None:
-                pb.update('comparing files', entry_count, num_entries)
-            if changes or include_unchanged:
-                if specific_files is not None:
-                    precise_file_ids.add(result.parent_id[1])
-                    changed_file_ids.append(result.file_id)
-                yield result
-            # Ensure correct behaviour for reparented/added specific files.
-            if specific_files is not None:
-                # Record output dirs
-                if result.kind[1] == 'directory':
-                    seen_dirs.add(result.file_id)
-                # Record parents of reparented/added entries.
-                if not result.versioned[0] or result.is_reparented():
-                    seen_parents.add(result.parent_id[1])
-        while all_unversioned:
-            # yield any trailing unversioned paths
-            unversioned_path = all_unversioned.popleft()
-            to_kind, to_executable, to_stat = \
-                self.target._comparison_data(fake_entry, unversioned_path[1])
-            yield TreeChange(
-                None, (None, unversioned_path[1]), True, (False, False),
-                (None, None),
-                (None, unversioned_path[0][-1]),
-                (None, to_kind),
-                (None, to_executable))
-        # Yield all remaining source paths
-        for path, from_entry in from_entries_by_dir:
-            file_id = from_entry.file_id
-            if file_id in to_paths:
-                # already returned
-                continue
-            to_path = self.find_target_path(path)
-            entry_count += 1
-            if pb is not None:
-                pb.update('comparing files', entry_count, num_entries)
-            versioned = (True, False)
-            parent = (from_entry.parent_id, None)
-            name = (from_entry.name, None)
-            from_kind, from_executable, stat_value = \
-                self.source._comparison_data(from_entry, path)
-            kind = (from_kind, None)
-            executable = (from_executable, None)
-            changed_content = from_kind is not None
-            # the parent's path is necessarily known at this point.
-            changed_file_ids.append(file_id)
-            yield TreeChange(
-                file_id, (path, to_path), changed_content, versioned, parent,
-                name, kind, executable)
-        changed_file_ids = set(changed_file_ids)
-        if specific_files is not None:
-            for result in self._handle_precise_ids(precise_file_ids,
-                                                   changed_file_ids):
-                yield result
-
-    @staticmethod
-    def _get_entry(tree, path):
-        """Get an inventory entry from a tree, with missing entries as None.
-
-        If the tree raises NotImplementedError on accessing .inventory, then
-        this is worked around using iter_entries_by_dir on just the file id
-        desired.
-
-        :param tree: The tree to lookup the entry in.
-        :param path: The path to look up
-        """
-        # No inventory available.
-        try:
-            iterator = tree.iter_entries_by_dir(specific_files=[path])
-            return next(iterator)[1]
-        except StopIteration:
-            return None
-
-    def _handle_precise_ids(self, precise_file_ids, changed_file_ids,
-                            discarded_changes=None):
-        """Fill out a partial iter_changes to be consistent.
-
-        :param precise_file_ids: The file ids of parents that were seen during
-            the iter_changes.
-        :param changed_file_ids: The file ids of already emitted items.
-        :param discarded_changes: An optional dict of precalculated
-            iter_changes items which the partial iter_changes had not output
-            but had calculated.
-        :return: A generator of iter_changes items to output.
-        """
-        # process parents of things that had changed under the users
-        # requested paths to prevent incorrect paths or parent ids which
-        # aren't in the tree.
-        while precise_file_ids:
-            precise_file_ids.discard(None)
-            # Don't emit file_ids twice
-            precise_file_ids.difference_update(changed_file_ids)
-            if not precise_file_ids:
-                break
-            # If the there was something at a given output path in source, we
-            # have to include the entry from source in the delta, or we would
-            # be putting this entry into a used path.
-            paths = []
-            for parent_id in precise_file_ids:
-                try:
-                    paths.append(self.target.id2path(parent_id))
-                except errors.NoSuchId:
-                    # This id has been dragged in from the source by delta
-                    # expansion and isn't present in target at all: we don't
-                    # need to check for path collisions on it.
-                    pass
-            for path in paths:
-                old_id = self.source.path2id(path)
-                precise_file_ids.add(old_id)
-            precise_file_ids.discard(None)
-            current_ids = precise_file_ids
-            precise_file_ids = set()
-            # We have to emit all of precise_file_ids that have been altered.
-            # We may have to output the children of some of those ids if any
-            # directories have stopped being directories.
-            for file_id in current_ids:
-                # Examine file_id
-                if discarded_changes:
-                    result = discarded_changes.get(file_id)
-                    source_entry = None
-                else:
-                    result = None
-                if result is None:
-                    try:
-                        source_path = self.source.id2path(file_id)
-                    except errors.NoSuchId:
-                        source_path = None
-                        source_entry = None
-                    else:
-                        source_entry = self._get_entry(
-                            self.source, source_path)
-                    try:
-                        target_path = self.target.id2path(file_id)
-                    except errors.NoSuchId:
-                        target_path = None
-                        target_entry = None
-                    else:
-                        target_entry = self._get_entry(
-                            self.target, target_path)
-                    result, changes = self._changes_from_entries(
-                        source_entry, target_entry, source_path, target_path)
-                else:
-                    changes = True
-                # Get this parents parent to examine.
-                new_parent_id = result.parent_id[1]
-                precise_file_ids.add(new_parent_id)
-                if changes:
-                    if (result.kind[0] == 'directory' and
-                            result.kind[1] != 'directory'):
-                        # This stopped being a directory, the old children have
-                        # to be included.
-                        if source_entry is None:
-                            # Reusing a discarded change.
-                            source_entry = self._get_entry(
-                                self.source, result.path[0])
-                        precise_file_ids.update(
-                            child.file_id
-                            for child in self.source.iter_child_entries(result.path[0]))
-                    changed_file_ids.add(result.file_id)
-                    yield result
+        raise NotImplementedError(self.iter_changes)
 
     def file_content_matches(
             self, source_path, target_path,
@@ -1200,13 +870,7 @@ class InterTree(InterObject):
         :return: path in target, or None if there is no equivalent path.
         :raise NoSuchFile: If the path doesn't exist in source
         """
-        file_id = self.source.path2id(path)
-        if file_id is None:
-            raise errors.NoSuchFile(path)
-        try:
-            return self.target.id2path(file_id, recurse=recurse)
-        except errors.NoSuchId:
-            return None
+        raise NotImplementedError(self.find_target_path)
 
     def find_source_path(self, path, recurse='none'):
         """Find the source tree path.
@@ -1215,13 +879,7 @@ class InterTree(InterObject):
         :return: path in source, or None if there is no equivalent path.
         :raise NoSuchFile: if the path doesn't exist in target
         """
-        file_id = self.target.path2id(path)
-        if file_id is None:
-            raise errors.NoSuchFile(path)
-        try:
-            return self.source.id2path(file_id, recurse=recurse)
-        except errors.NoSuchId:
-            return None
+        raise NotImplementedError(self.find_source_path)
 
     def find_target_paths(self, paths, recurse='none'):
         """Find target tree paths.
@@ -1248,9 +906,6 @@ class InterTree(InterObject):
         return ret
 
 
-InterTree.register_optimiser(InterTree)
-
-
 def find_previous_paths(from_tree, to_tree, paths, recurse='none'):
     """Find previous tree paths.
 
@@ -1288,6 +943,8 @@ def get_canonical_path(tree, path, normalize):
     cur_path = ''
     bit_iter = iter(path.split("/"))
     for elt in bit_iter:
+        if not elt:
+            continue
         lelt = normalize(elt)
         new_path = None
         try:
